import io
import zipfile
from lxml import etree

from . import (
    Mimetype,
    Preferences,
    Styles,
    Tags,
    Story,
    StoryTable,
    StoryList,
    Spread,
    Designmap,
    Graphic
)


class Converter:
    """
    IDML Converter.

    Format items to `IDML <https://fileinfo.com/extension/idml>`
    """

    ZIP_COMPRESSION = zipfile.ZIP_STORED
    PAGE_MARGIN_TOP = 50
    PAGE_MARGIN_BOTTOM = 50
    PAGE_MARGIN_LEFT = 50
    PAGE_MARGIN_RIGHT = 50
    DOCUMENT_PAGE_HEIGHT = 841.8897637776
    DOCUMENT_PAGE_WIDTH = 595.2755905488
    DOCUMENT_PAGE_INNER_WIDTH = DOCUMENT_PAGE_WIDTH - PAGE_MARGIN_LEFT - PAGE_MARGIN_RIGHT

    def __init__(self):
        self._idml_bytes_buffer = None
        self._in_memory_zip = None
        self._package = []
        self._counter = {}

    def create_idml(self, article):
        """
        Creates idml file as in-memory zip container.

        IDML file is a zip container and it inludes the next files and folders:
         - mimetype
         - designmap.xml:
            Is the key to all of the other filles that appear within the IDML package.
         - XML:
            Folder contains XML elements and settings used in the InDesign document.
         - XML/Tags.xml:
            Contains the XML tag defnitions stored in the InDesign document.
         - Resources:
            Folder contains elements that are commonly used by other filles within the document,
            such as colors, fonts, and paragraph styles.
         - Resources/Graphic.xml:
            Contains the inks, colors, swatches, gradients, mixed inks, mixed ink groups, tints,
            and stroke styles contained in the document.
         - Resources/Preferences.xml:
            Contains representations of all of the document preferences.
         - Resources/Styles.xml:
            Contains all of the paragraph, character, object, cell, table, and TOC styles used in the document.
         - Spreads:
            Folder contains the XML  les representing the spreads in the document.
         - Spreads/Spread_spread_*.xml:
            Each spread contains all of the page items (rectangles, ellipses, graphic lines, polygons, groups,
            buttons, and text frames) that appear on the pages of the spread.
         - Stories:
            Folder contains all of the stories in the InDesign document.
         - Stories/Story_story_*.xml:
            Represents the contents of a single story and all of the formatting attributes applied to the
            text in the story.

        :param dict article: article data
        :return bytes: idml file as a bytes
        """
        self._counter = {'spread': 0, 'page': 0, 'story': 0, 'textframe': 0}
        self._init_zip_container()
        self._package = []
        self._package.append(Mimetype())
        self._package.append(Styles())
        self._package.append(Graphic())
        self._package.append(
            Preferences(attributes={
                'DocumentPreference': {
                    'PageHeight': str(self.DOCUMENT_PAGE_HEIGHT),
                    'PageWidth': str(self.DOCUMENT_PAGE_WIDTH),
                    'PagesPerDocument': '1',
                    'FacingPages': 'false'}
            })
        )
        self._package.append(
            Tags(attributes={
                'Headline': {'TagColor': 'Orange'},
                'Byline': {'TagColor': 'Orange'},
                'Heading1': {'TagColor': 'Red'},
                'Heading2': {'TagColor': 'Red'},
                'Heading3': {'TagColor': 'Red'},
                'Heading4': {'TagColor': 'Red'},
                'Heading5': {'TagColor': 'Red'},
                'Heading6': {'TagColor': 'Red'},
                'NormalParagraph': {'TagColor': 'Green'},
                'Blockquote': {'TagColor': 'Blue'},
                'Preformatted': {'TagColor': 'Black'},
                'Table': {'TagColor': 'Yellow'},
                'UnorderedList': {'TagColor': 'Pink'},
                'OrderedList': {'TagColor': 'Pink'}
            })
        )
        self._create_stories(article)
        self._create_spreads()
        self._create_designmap()
        self._write_package()
        self._in_memory_zip.close()

        return self._idml_bytes_buffer.getvalue()

    def _init_zip_container(self):
        """
        Creates zipfile with `x` (exclusively create and write a new file) mode.
        BytesIO file-like object is used for zipfile file reference.
        """
        self._idml_bytes_buffer = io.BytesIO()
        self._in_memory_zip = zipfile.ZipFile(
            self._idml_bytes_buffer,
            mode='x',
            compression=self.ZIP_COMPRESSION
        )

    def _write_package(self):
        """
        Write files into zip container.

        ``self._package`` includes only objects based on
        ``<class 'superdesk.publish.formatters.idml_formatter.package.base.BasePackageElement'>``,
        they know how to render itself (file content) and how to name itself.
        """
        for item in self._package:
            self._in_memory_zip.writestr(
                item.filename,
                item.render()
            )

    def _create_stories(self, article):
        """
        Walk thrrough `article['body_html']` and creates a Story/StoryTable/StoryList instances for each element.
        :param dict article: article data
        """

        body_html = article['body_html']

        if article.get('byline'):
            byline = etree.Element('byline')
            byline.text = article.get('byline')
            body_html = etree.tostring(byline, pretty_print=False).decode('utf-8') + body_html
        if article.get('headline'):
            headline = etree.Element('headline')
            headline.text = article.get('headline')
            body_html = etree.tostring(headline, pretty_print=False).decode('utf-8') + body_html

        parser = etree.HTMLParser(recover=True, remove_blank_text=True)
        root = etree.fromstring(body_html, parser)
        body = root.find('body')

        for element in body:
            if element.tag in ('p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'blockquote', 'pre', 'headline', 'byline'):
                # create text story
                self._package.append(
                    Story(
                        self._next_story_id(),
                        element,
                        attributes={
                            'ParagraphStyleRange': Story.BLOCK_TAGS_MAPPING[element.tag]['ParagraphStyleRange']
                        },
                        markup_tag=Story.BLOCK_TAGS_MAPPING[element.tag]['markup_tag']
                    )
                )
            elif element.tag in ('ul', 'ol'):
                # create list story
                self._package.append(
                    StoryList(
                        self._next_story_id(),
                        element,
                        attributes={
                            'ParagraphStyleRange': Story.BLOCK_TAGS_MAPPING[element.tag]['ParagraphStyleRange']
                        },
                        markup_tag=Story.BLOCK_TAGS_MAPPING[element.tag]['markup_tag']
                    )
                )
            elif element.tag == 'table':
                # create table story
                self._package.append(
                    StoryTable(
                        self._next_story_id(),
                        element,
                        self.DOCUMENT_PAGE_INNER_WIDTH,
                        attributes={
                            'Table': Story.BLOCK_TAGS_MAPPING[element.tag]['Table']
                        },
                        markup_tag=Story.BLOCK_TAGS_MAPPING[element.tag]['markup_tag']
                    )
                )

    def _create_spreads(self):
        """
        Walk through all stories, create a spread(s) and place stories into strpead(s).
        """
        active_spread = self._create_spread_with_page()

        # walk through packages and place them into spread->page
        for package in self._package:
            _type = type(package)
            if _type in (Story, StoryTable, StoryList):
                # guess height for a text frame (yes, table and lists are also textframes)
                text_frame_height = _type.guess_height(package, self.DOCUMENT_PAGE_INNER_WIDTH)

                # if guessed height for text frame is more than page height
                if text_frame_height > active_spread.page_inner_height:
                    text_frame_height = active_spread.page_inner_height

                # check if textframe will fit current page
                if not active_spread.check_if_fits(text_frame_height):
                    active_spread = self._create_spread_with_page()

                # place text frame
                active_spread.place_textframe(
                    height=text_frame_height,
                    attributes={
                        'TextFrame': {
                            'Self': self._next_textframe_id(),
                            'ParentStory': package.self_id,
                        }
                    }
                )

    def _create_designmap(self):
        """
        Create `Designmap` instance add it to the package.
        """
        designmap = Designmap()

        # let's fill designmap
        # order plays a big role
        for _type in (Graphic, Styles, Preferences, Tags, Spread, Story, StoryTable, StoryList):
            for obj in [i for i in self._package if i.__class__ is _type]:
                designmap.add_pkg(_type.__name__, obj.filename)

        for _type in (Story, StoryTable, StoryList):
            for obj in [i for i in self._package if i.__class__ is _type]:
                designmap.add_hyperlinks(obj.links)

        self._package.append(designmap)

    def _create_spread_with_page(self):
        """
        Create spread and put one page inside.
        :return Spread: spread instance
        """
        spread = Spread(
            self._next_spread_id(),
            document_page_width=self.DOCUMENT_PAGE_WIDTH,
            document_page_height=self.DOCUMENT_PAGE_HEIGHT,
        )
        self._package.append(spread)

        # create page
        page_id = self._next_page_id()
        spread.add_page(
            {
                'Page': {
                    'Self': page_id,
                    'Name': page_id,
                    'UseMasterGrid': 'false'
                },
                'MarginPreference': {
                    'Top': str(self.PAGE_MARGIN_TOP),
                    'Bottom': str(self.PAGE_MARGIN_BOTTOM),
                    'Left': str(self.PAGE_MARGIN_LEFT),
                    'Right': str(self.PAGE_MARGIN_RIGHT)
                },
            }
        )

        return spread

    def _next_story_id(self):
        """
        Generate a story id.
        :return str: story id
        """
        self_id = 'story_{}'.format(self._counter['story'])
        self._counter['story'] += 1
        return self_id

    def _next_spread_id(self):
        """
        Generate a spread id.
        :return str: spread id
        """
        self_id = 'spread_{}'.format(self._counter['spread'])
        self._counter['spread'] += 1
        return self_id

    def _next_page_id(self):
        """
        Generate a page id.
        :return str: page id
        """
        self_id = 'page_{}'.format(self._counter['page'])
        self._counter['page'] += 1
        return self_id

    def _next_textframe_id(self):
        """
        Generate a textframe id.
        :return str: textframe id
        """
        self_id = 'textframe_{}'.format(self._counter['textframe'])
        self._counter['textframe'] += 1
        return self_id



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

from .enums import ConceptNature


CONCEPT_ITEMS_PRIVELEGE = 'concept_items'


class ConceptItemsResource(Resource):
    """Concept item schema"""

    schema = {
        'name': {
            'type': 'string',
            'required': True,
            'empty': False
        },
        'group_id': {
            'type': 'string',
            'required': False,
            'empty': False,
            'readonly': False
        },
        'definition_text': {
            'type': 'string',
            'required': False,
            'empty': False,
            'readonly': True
        },
        'definition_html': {
            'type': 'string',
            'required': True,
            'empty': False
        },
        'language': {
            'type': 'string',
            'required': True,
            'empty': False
        },
        'labels': {
            'type': 'list',
            'schema': {
                'type': 'string'
            },
            'unique_list': True
        },
        # http://cv.iptc.org/newscodes/cpnature/
        'cpnat_type': {
            'type': 'string',
            'required': True,
            'allowed': ConceptNature.values()
        },
        # https://iptc.org/std/NewsML-G2/guidelines/#more-real-world-entities
        'properties': {
            'type': 'dict',
            'required': False
        },
        'created_by': Resource.rel('users', embeddable=False, readonly=True),
        'updated_by': Resource.rel('users', embeddable=False, readonly=True),
    }
    privileges = {
        'POST': CONCEPT_ITEMS_PRIVELEGE,
        'PATCH': CONCEPT_ITEMS_PRIVELEGE,
        'PUT': CONCEPT_ITEMS_PRIVELEGE,
        'DELETE': CONCEPT_ITEMS_PRIVELEGE
    }
    item_url = 'regex("[a-f0-9]{24}")'
    resource_methods = ['GET', 'POST']
    item_methods = ['GET', 'PATCH', 'PUT', 'DELETE']
    query_objectid_as_string = False
    mongo_indexes = {
        'name_collation': ([('name', 1)], {'collation': {'locale': 'en', 'strength': 1}}),
        'definition_text_collation': ([('definition_text', 1), ], {'collation': {'locale': 'en', 'strength': 1}}),
        'group_id': ([('group_id', 1), ('language', 1)], {'unique': True})
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json
import datetime
from unittest import mock

from superdesk.publish import init_app
from superdesk.publish.formatters.email_formatter import EmailFormatter
from superdesk.tests import TestCase
from superdesk.utc import utc


@mock.patch('superdesk.publish.subscribers.SubscribersService.generate_sequence_number', lambda self, subscriber: 1)
class EmailFormatterTest(TestCase):
    def setUp(self):
        self.formatter = EmailFormatter()
        # self.base_formatter = Formatter()
        init_app(self.app)

    def test_formatter(self):
        article = {
            'source': 'AAP',
            'headline': 'This is a test headline',
            'abstract': 'Can of beans',
            'byline': 'joe',
            'ednote': 'Very good story',
            'dateline': {'text': 'BERN, July 13  -'},
            'slugline': 'slugline',
            'anpa_take_key': 'take',
            'subject': [{'qcode': '02011001'}],
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>The story body of the story so far</p>',
            'word_count': '1',
            'priority': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}],
            'genre': [],
            'sign_off': 'aa/bb'
        }

        article['versioncreated'] = datetime.datetime(year=2015, month=1, day=30, hour=2, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertEqual(item['message_subject'], 'This is a test headline')
        self.assertEqual(item['message_html'], '<html>\n<body>\n<h1>VIC:&nbsp;This is a test headline</h1>\n'
                                               'Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline'
                                               '</b>&nbsp;take&nbsp;\n'
                                               '<hr>\n<br><font color="red">Very good story</font>'
                                               '<br>\n<i>Can of beans</i>\n<br>joe\n<br>\n'
                                               '<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n'
                                               'AAP&nbsp;aa/bb\n\n</body>\n</html>')
        self.assertEqual(item['message_text'], 'VIC: This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n'
                                               'slugline take\nVery good story\n------------------------------------'
                                               '----------------------\nCan of beans\n\njoe\nBERN, July 13  - The story'
                                               ' body of the story so far\n\nAAP aa/bb\n')

    def test_preserved_formatter(self):
        article = {
            'source': 'AAP',
            'headline': 'This is a test headline',
            'abstract': 'Can of beans',
            'byline': 'joe',
            'dateline': {'text': 'BERN, July 13  -'},
            'slugline': 'slugline',
            'anpa_take_key': 'take',
            'subject': [{'qcode': '02011001'}],
            'format': 'preserved',
            'type': 'text',
            'body_html': 'The story body of the story so far',
            'word_count': '1',
            'priority': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}],
            'genre': [],
            'sign_off': 'aa/bb'
        }

        article['versioncreated'] = datetime.datetime(year=2015, month=1, day=30, hour=2, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertEqual(item['message_subject'], 'This is a test headline')
        self.assertEqual(item['message_html'], None)

    def test_no_place_formatter(self):
        article = {
            'source': 'AAP',
            'headline': 'This is a test headline',
            'abstract': 'Can of beans',
            'byline': 'joe',
            'ednote': 'Very good story',
            'dateline': {'text': 'BERN, July 13  -'},
            'slugline': 'slugline',
            'anpa_take_key': 'take',
            'subject': [{'qcode': '02011001'}],
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>The story body of the story so far</p>',
            'word_count': '1',
            'priority': 1,
            'genre': [],
            'sign_off': 'aa/bb'
        }

        article['versioncreated'] = datetime.datetime(year=2015, month=1, day=30, hour=2, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertEqual(item['message_subject'], 'This is a test headline')
        self.assertEqual(item['message_html'], '<html>\n<body>\n<h1>This is a test headline</h1>\n'
                                               'Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline'
                                               '</b>&nbsp;take&nbsp;\n'
                                               '<hr>\n<br><font color="red">Very good story</font>'
                                               '<br>\n<i>Can of beans</i>\n<br>joe\n<br>\n'
                                               '<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n'
                                               'AAP&nbsp;aa/bb\n\n</body>\n</html>')
        self.assertEqual(item['message_text'], 'This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n'
                                               'slugline take\nVery good story\n------------------------------------'
                                               '----------------------\nCan of beans\n\njoe\nBERN, July 13  - The story'
                                               ' body of the story so far\n\nAAP aa/bb\n')

    def test_none_place_formatter(self):
        article = {
            'source': 'AAP',
            'headline': 'This is a test headline',
            'abstract': 'Can of beans',
            'byline': 'joe',
            'ednote': 'Very good story',
            'dateline': {'text': 'BERN, July 13  -'},
            'slugline': 'slugline',
            'anpa_take_key': 'take',
            'subject': [{'qcode': '02011001'}],
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>The story body of the story so far</p>',
            'word_count': '1',
            'priority': 1,
            'place': None,
            'genre': [],
            'sign_off': 'aa/bb'
        }

        article['versioncreated'] = datetime.datetime(year=2015, month=1, day=30, hour=2, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertEqual(item['message_subject'], 'This is a test headline')
        self.assertEqual(item['message_html'], '<html>\n<body>\n<h1>This is a test headline</h1>\n'
                                               'Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline'
                                               '</b>&nbsp;take&nbsp;\n'
                                               '<hr>\n<br><font color="red">Very good story</font>'
                                               '<br>\n<i>Can of beans</i>\n<br>joe\n<br>\n'
                                               '<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n'
                                               'AAP&nbsp;aa/bb\n\n</body>\n</html>')
        self.assertEqual(item['message_text'], 'This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n'
                                               'slugline take\nVery good story\n------------------------------------'
                                               '----------------------\nCan of beans\n\njoe\nBERN, July 13  - The'
                                               ' story body of the story so far\n\nAAP aa/bb\n')

    def test_none_takekey_ednote(self):
        article = {
            'source': 'AAP',
            'headline': 'This is a test headline',
            'abstract': 'Can of beans',
            'byline': 'joe',
            'ednote': None,
            'dateline': {'text': 'BERN, July 13  -'},
            'slugline': 'slugline',
            'anpa_take_key': None,
            'subject': [{'qcode': '02011001'}],
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>The story body of the story so far</p>',
            'word_count': '1',
            'priority': 1,
            'place': None,
            'genre': [],
            'sign_off': 'aa/bb'
        }

        article['versioncreated'] = datetime.datetime(year=2015, month=1, day=30, hour=2, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertEqual(item['message_subject'], 'This is a test headline')
        self.assertEqual(item['message_html'], '<html>\n<body>\n<h1>This is a test headline</h1>\n'
                                               'Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline'
                                               '</b>&nbsp;\n<hr>\n'
                                               '\n<i>Can of beans</i>\n<br>joe\n<br>\n'
                                               '<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n'
                                               'AAP&nbsp;aa/bb\n\n</body>\n</html>')
        self.assertEqual(item['message_text'], 'This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n'
                                               'slugline \n\n------------------------------------'
                                               '----------------------\nCan of beans\n\njoe\nBERN, July 13  - The story'
                                               ' body of the story so far\n\nAAP aa/bb\n')

    def test_subject_cyrilic(self):
        article = {'headline': 'Неправильная музыка Джамала Али'}
        seq, doc = self.formatter.format(article, {'name': 'Test'})[0]
        item = json.loads(doc)
        self.assertEqual(article['headline'], item['message_subject'])

    def test_paragraphs(self):
        """Test that paragraphs (block elements) are followed by line feed in text version

        SDESK-824 regression test
        """
        article = {
            'source': 'AAP',
            'anpa_take_key': 'take',
            'subject': [{'qcode': '02011001'}],
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>paragraph 1</p><br><br><p>paragraph 2</p><p>paragraph 3</p>',
            'word_count': '1',
            'priority': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}],
            'genre': [],
            'sign_off': 'aa/bb'
        }
        article['versioncreated'] = datetime.datetime(year=2017, month=2, day=24, hour=16, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]

        item = json.loads(doc)
        self.assertEqual(item['message_text'], 'VIC: \nPublished At : Fri Feb 24 17:40:56 2017\n'
                                               ' take\n\n--------------------------------------'
                                               '--------------------\n\n\n\nparagraph 1\n\n\n'
                                               'paragraph 2\nparagraph 3\n\nAAP aa/bb\n')

    def test_dateline_html(self):
        """Check that message_html output is producted even without a dateline

        SDESK-836 regression test
        """
        article = {
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>some HTML</p>'}
        _, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertIsNotNone(item['message_html'])

    def test_featuremedia(self):
        article = {
            'source': 'AAP',
            'anpa_take_key': 'take',
            'subject': [{'qcode': '02011001'}],
            'format': 'HTML',
            'type': 'text',
            'body_html': '<p>Test</p>',
            'word_count': '1',
            'priority': 1,
            'place': [{'qcode': 'VIC', 'name': 'VIC'}],
            'genre': [],
            'sign_off': 'aa/bb',
            'associations': {
                'featuremedia': {
                    'archive_description': 'A woman crosses Bourke Street Mall',
                    'renditions': {
                        'viewImage': {
                            'poi': {
                                'y': 257,
                                'x': 454
                            },
                            'href': 'http://localhost:5000/api/upload-raw/5c11ece81d41c89113ed202b?_schema=http',
                            'media': '5c11ece81d41c89113ed202b',
                            'height': 390,
                            'mimetype': 'image/jpeg',
                            'width': 640
                        }
                    }
                }
            }
        }
        article['versioncreated'] = datetime.datetime(year=2017, month=2, day=24, hour=16, minute=40, second=56,
                                                      tzinfo=utc)
        seq, doc = self.formatter.format(article, {'name': 'Test Subscriber'})[0]
        item = json.loads(doc)
        self.assertEqual(item['renditions']['viewImage']['media'], '5c11ece81d41c89113ed202b')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from flask import json

from apps.io.tests import setup_providers, teardown_providers
from apps.prepopulate.app_populate import AppPopulateCommand
from apps.prepopulate.app_initialize import AppInitializeWithDataCommand
from superdesk import tests
from superdesk.factory.app import get_app
from superdesk.tests import setup_auth_user
from superdesk.tests.mocks import TestSearchProvider
from superdesk.tests.steps import get_macro_path


readonly_fields = ['display_name', 'password', 'phone', 'first_name', 'last_name']
LDAP_SERVER = os.environ.get('LDAP_SERVER')


def setup_before_all(context, config, app_factory):
    """
    Keep it for backwards compatibility.

    TODO: it needs to be cleaned on each superdesk repo.
    """
    setup_before_all.config = config
    setup_before_all.app_factory = app_factory


def setup_before_scenario(context, scenario, config, app_factory):
    if scenario.status != 'skipped' and 'notesting' in scenario.tags:
        config['SUPERDESK_TESTING'] = False

    tests.setup(context, config, app_factory, bool(config))

    context.headers = [
        ('Content-Type', 'application/json'),
        ('Origin', 'localhost')
    ]

    if 'dbauth' in scenario.tags and LDAP_SERVER:
        scenario.mark_skipped()

    if 'ldapauth' in scenario.tags and not LDAP_SERVER:
        scenario.mark_skipped()

    if 'alchemy' in scenario.tags and not context.app.config.get('KEYWORDS_KEY_API'):
        scenario.mark_skipped()

    if 'clean_snapshots' in scenario.tags:
        tests.use_snapshot.cache.clear()

    setup_search_provider(context.app)

    if scenario.status != 'skipped' and 'auth' in scenario.tags:
        setup_auth_user(context)

    if scenario.status != 'skipped' and 'provider' in scenario.tags:
        setup_providers(context)

    if scenario.status != 'skipped' and 'vocabulary' in scenario.tags:
        with context.app.app_context():
            cmd = AppPopulateCommand()
            filename = os.path.join(os.path.abspath(os.path.dirname("features/steps/fixtures/")), "vocabularies.json")
            cmd.run(filename)

    if scenario.status != 'skipped' and 'content_type' in scenario.tags:
        with context.app.app_context():
            cmd = AppPopulateCommand()
            filename = os.path.join(os.path.abspath(os.path.dirname("features/steps/fixtures/")), "content_types.json")
            cmd.run(filename)

    if scenario.status != 'skipped' and 'notification' in scenario.tags:
        tests.setup_notification(context)

    if scenario.status != 'skipped' and 'app_init' in scenario.tags:
        with context.app.app_context():
            command = AppInitializeWithDataCommand()
            command.run()


def before_all(context):
    # https://pythonhosted.org/behave/api.html#logging-setup
    context.config.setup_logging()


def before_feature(context, feature):
    config = getattr(setup_before_all, 'config', None)
    if config is not None:
        app_factory = setup_before_all.app_factory
    else:
        # superdesk-aap don't use "setup_before_all" already
        config = getattr(setup_before_scenario, 'config', None)
        app_factory = getattr(setup_before_scenario, 'app_factory', None)
    config = config or {}
    app_factory = app_factory or get_app

    # set the MAX_TRANSMIT_RETRY_ATTEMPT to zero so that transmit does not retry
    config['MAX_TRANSMIT_RETRY_ATTEMPT'] = 0
    os.environ['BEHAVE_TESTING'] = '1'
    tests.setup(context, config, app_factory=app_factory)

    if 'tobefixed' in feature.tags:
        feature.mark_skipped()

    if 'dbauth' in feature.tags and LDAP_SERVER:
        feature.mark_skipped()

    if 'ldapauth' in feature.tags and not LDAP_SERVER:
        feature.mark_skipped()


def before_scenario(context, scenario):
    config = {}
    setup_before_scenario(context, scenario, config, app_factory=get_app)


def after_scenario(context, scenario):
    if 'provider' in scenario.tags:
        teardown_providers(context)

    if 'notification' in scenario.tags:
        tests.teardown_notification(context)

    if 'clean' in scenario.tags:
        try:
            os.remove(get_macro_path('behave_macro.py'))
            os.remove(get_macro_path('validate_headline_macro.py'))
        except Exception:
            pass


def before_step(context, step):
    if LDAP_SERVER and step.text:
        try:
            step_text_json = json.loads(step.text)
            step_text_json = {k: step_text_json[k] for k in step_text_json.keys() if k not in readonly_fields} \
                if isinstance(step_text_json, dict) else \
                [{k: json_obj[k] for k in json_obj.keys() if k not in readonly_fields} for json_obj in step_text_json]

            step.text = json.dumps(step_text_json)
        except Exception:
            pass


def setup_search_provider(app):
    from apps.search_providers import register_search_provider, allowed_search_providers
    if 'testsearch' not in allowed_search_providers:
        register_search_provider('testsearch', provider_class=TestSearchProvider)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from eve.utils import ParsedRequest

from . import init_app
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from superdesk.tests import TestCase


def resource_listener(resource, docs):
    for doc in docs['_items']:
        doc['_resource_listener'] = resource


def ingest_listener(docs):
    for doc in docs['_items']:
        doc['_ingest_listener'] = 1


class SearchServiceTestCase(TestCase):

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('ingest', [{}])
            self.app.data.insert('archive', [{'_id': '456', 'task': {'desk': 1}, ITEM_STATE: CONTENT_STATE.PROGRESS},
                                             {'_id': '911', 'task': {'desk': None, 'user': 'test'},
                                              ITEM_STATE: CONTENT_STATE.PROGRESS},
                                             {'_id': '123', 'task': {'desk': 1}, ITEM_STATE: CONTENT_STATE.PUBLISHED},
                                             {'_id': '786', 'task': {'desk': 1}, ITEM_STATE: CONTENT_STATE.PUBLISHED}])
            self.app.data.insert('published', [{'item_id': '786', 'task': {'desk': 1},
                                                ITEM_STATE: CONTENT_STATE.PUBLISHED}])
            self.app.data.insert('archived', [{'item_id': '123', 'task': {'desk': 1},
                                               ITEM_STATE: CONTENT_STATE.PUBLISHED}])
            init_app(self.app)
            self.app.on_fetched_resource += resource_listener
            self.app.on_fetched_resource_ingest += ingest_listener

    def test_query_post_processing(self):
        with self.app.app_context():
            docs = self.app.data.find('search', None, None)
            self.assertEquals(4, docs.count())

            ingest_docs = [doc for doc in docs if doc['_type'] == 'ingest']
            self.assertEquals('ingest', ingest_docs[0]['_resource_listener'])
            self.assertEquals(1, ingest_docs[0]['_ingest_listener'])

            archive_docs = [doc for doc in docs if doc['_type'] == 'archive']
            self.assertNotIn('_ingest_listener', archive_docs[0])

    def test_using_repo_request_attribute(self):
        with self.app.app_context():
            req = ParsedRequest()
            req.args = {'repo': 'ingest'}
            docs = self.app.data.find('search', req, None)
            self.assertEquals(1, docs.count())
            self.assertEquals('ingest', docs[0]['_type'])

    def test_it_filters_out_private_content(self):
        with self.app.app_context():
            self.app.data.insert('archive', [{'task': {'desk': None}}, {'task': {}}])
            cursor = self.app.data.find('search', None, None)
            self.assertEquals(4, cursor.count())

    def test_it_includes_published_content(self):
        with self.app.app_context():
            cursor = self.app.data.find('search', None, None)
            self.assertEquals(4, cursor.count())

    def test_it_excludes_published_content(self):
        with self.app.app_context():
            req = ParsedRequest()
            req.args = {'repo': 'archive'}
            docs = self.app.data.find('search', req, None)
            self.assertEquals(1, docs.count())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import request

from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.metadata.utils import item_url
from apps.archive.archive import SOURCE as ARCHIVE
from superdesk.workflow import is_workflow_state_transition_valid
from eve.utils import config
from apps.auth import get_user
import superdesk
from flask_babel import _


class CopyResource(Resource):
    endpoint_name = 'copy'
    resource_title = endpoint_name

    # Eve needs schema definition even if all the properties are optional.
    schema = {
        'desk': Resource.rel('desks', False, required=False)
    }

    url = 'archive/<{0}:guid>/copy'.format(item_url)

    resource_methods = ['POST']
    item_methods = []


class CopyService(BaseService):
    def create(self, docs, **kwargs):
        guid_of_item_to_be_copied = request.view_args['guid']

        guid_of_copied_items = []

        for doc in docs:
            archive_service = get_resource_service(ARCHIVE)

            archived_doc = archive_service.find_one(req=None, _id=guid_of_item_to_be_copied)
            if not archived_doc:
                raise SuperdeskApiError.notFoundError(_(
                    'Fail to found item with guid: {guid}').format(guid=guid_of_item_to_be_copied))

            current_desk_of_item = archived_doc.get('task', {}).get('desk')
            if current_desk_of_item:
                raise SuperdeskApiError.preconditionFailedError(message=_('Copy is not allowed on items in a desk.'))

            if not is_workflow_state_transition_valid('copy', archived_doc[ITEM_STATE]):
                raise InvalidStateTransitionError()

            new_guid = archive_service.duplicate_content(archived_doc)
            guid_of_copied_items.append(new_guid)

        if kwargs.get('notify', True):
            user = get_user()
            push_notification('item:copy', copied=1, user=str(user.get(config.ID_FIELD, '')))

        return guid_of_copied_items


superdesk.workflow_action(
    name='copy',
    include_states=[CONTENT_STATE.DRAFT],
    privileges=['archive']
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


class InvalidFilter(Exception):
    def __init__(self, filter, operation):
        self.filter = filter
        self.operation = operation

    def __str__(self):
        return 'Invalid filter on %s: %s' % (self.operation, filter)


class InvalidEtag(Exception):
    def __str__(self):
        return 'Client and server etags don\'t match'


class Validator():
    def validate(self, doc):
        raise NotImplementedError()


class ValidationError():
    def __init__(self, errors):
        self.errors = errors


class BaseModel():
    """This is a basic interface for defining models.

    The only requirement is
    to implement the name method that uniquely identifies a model.
    """

    def __init__(self, resource, data_layer, schema, validator):
        """Runs on baseModel init

        :param resource: str The resource name
        :param data_layer: object The object that implements the database operations.
        :param schema: dict The model schema definition
        :param validator: object The object that validates the model data.
        """
        self.resource = resource
        self.data_layer = data_layer
        self.schema = schema
        self.validator = validator

    @classmethod
    def name(cls):
        raise NotImplementedError()

    def on_create(self, docs):
        """Method called before the create operation.

        :param docs: list of docs to be inserted
        """

    def on_created(self, docs):
        """Method called after the create operation.

        :param docs: list of inserted docs
        """

    def on_update(self, doc, orig):
        """Method called before the update/replace operations.

        :param doc: the document updates
        :param orig: the original document before the update
        """

    def on_updated(self, doc, orig):
        """Method called after the update/replace operations.

        :param doc: the document updates
        :param orig: the original document before the update
        """

    def on_delete(self, docs):
        """Method called before the delete operation.

        :param docs: list of docs to be deleted
        """

    def on_deleted(self, docs):
        """Method called after the delete operation.

        :param docs: list of deleted docs
        """

    def etag(self, doc):
        """
        Return the document etag.
        """
        return self.data_layer.etag(doc)

    def validate_etag(self, doc, etag):
        """Validates the given etag for the given document.

        Raises InvalidEtag if they don't match.
        """
        if self.etag(doc) != etag:
            raise InvalidEtag()

    def find_one(self, filter, projection=None):
        """Return one document selected based on the given filter.

        :param filter: dict
        :param projection: dict
        """
        return self.data_layer.find_one(self.resource, filter, projection)

    def find(self, filter, projection=None, **options):
        """Return a list of documents selected based on the given filter.

        :param filter: dict
        :param projection: dict
        :param options: dict
        """
        return self.data_layer.find(self.resource, filter, projection, **options)

    def create(self, docs):
        """Insert a list of documents.

        :param docs: list
        """
        self.validate(docs)
        self.on_create(docs)
        res = self.data_layer.create(self.resource, docs)
        self.on_created(docs)
        return res

    def update(self, filter, doc, etag=None):
        """Update one document selected based on the given filter.

        :param filter: dict
        :param doc: dict
        """
        self.validate(doc)
        orig = self.find_one(filter)
        if not orig:
            raise InvalidFilter(filter, 'update')
        if etag:
            self.validate_etag(orig, etag)
        self.on_update(doc, orig)
        res = self.data_layer.update(self.resource, filter, doc)
        self.on_updated(doc, orig)
        return res

    def replace(self, filter, doc, etag=None):
        """Replace the content of one document selected based on the given filter.

        :param filter: dict
        :param doc: dict
        """
        self.validate(doc)
        orig = self.find(filter)
        if orig.count() != 1:
            raise InvalidFilter(filter, 'replace')

        orig = list(orig)

        if etag:
            self.validate_etag(orig, etag)
        self.on_update(doc, orig[0])
        res = self.data_layer.replace(self.resource, filter, doc)
        self.on_updated(doc, orig[0])
        return res

    def delete(self, filter):
        """Delete one document selected based on the given filter.

        :param filter: dict
        """
        orig = self.find_one(filter)
        if not orig:
            return
        self.on_delete(orig)
        res = self.data_layer.delete(self.resource, filter)
        self.on_deleted(orig)
        return res

    def validate(self, docs):
        """Validate a document or a list of documents.

        Raise ValidationError on errors

        :param docs: list of documents
        """
        errs = []
        if not isinstance(docs, list):
            docs = [docs]
        for doc in docs:
            err = self.validator.validate(doc)
            if errs:
                errs.append(err)
        if errs:
            raise ValidationError(errs)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime

from superdesk.etree import etree
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.iptc import subject_codes
from superdesk.errors import ParserError
from superdesk.metadata.item import ITEM_TYPE
from superdesk.metadata.item import CONTENT_TYPE
from superdesk.utc import utc


class NewsMLOneFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse if the feed is in NewsML 1.2 format.
    """

    NAME = 'newsml12'

    label = 'News ML 1.2 Parser'

    def can_parse(self, xml):
        return xml.tag == 'NewsML' and xml.get('Version', '') == '1.2'

    def parse(self, xml, provider=None):
        item = {}
        try:
            self.root = xml
            parsed_el = xml.find('NewsItem/NewsComponent/AdministrativeMetadata/Source')
            if parsed_el is not None:
                item['original_source'] = parsed_el.find('Party').get('FormalName', '')

            parsed_el = xml.find('NewsEnvelope/TransmissionId')
            if parsed_el is not None:
                item['ingest_provider_sequence'] = parsed_el.text

            parsed_el = xml.find('NewsEnvelope/Priority')
            item['priority'] = self.map_priority(parsed_el.text if parsed_el is not None else None)

            self.parse_news_identifier(item, xml)
            self.parse_newslines(item, xml)
            self.parse_news_management(item, xml)

            parsed_el = xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/Language')
            if parsed_el is not None:
                language = self.parse_attributes_as_dictionary(parsed_el)
                item['language'] = language[0]['FormalName'] if len(language) else ''

            keywords = xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/Property')
            item['keywords'] = self.parse_attribute_values(keywords, 'Keyword')

            subjects = xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectDetail')
            subjects += xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectMatter')
            subjects += xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/Subject')

            item['subject'] = self.format_subjects(subjects)

            # item['ContentItem'] = self.parse_attributes_as_dictionary(
            #    tree.find('NewsItem/NewsComponent/ContentItem'))
            # item['Content'] = etree.tostring(
            # tree.find('NewsItem/NewsComponent/ContentItem/DataContent/nitf/body/body.content'))

            item['body_html'] = etree.tostring(
                xml.find('NewsItem/NewsComponent/ContentItem/DataContent/nitf/body/body.content'),
                encoding='unicode').replace('<body.content>', '').replace('</body.content>', '')

            parsed_el = xml.findall('NewsItem/NewsComponent/ContentItem/Characteristics/Property')
            characteristics = self.parse_attribute_values(parsed_el, 'Words')
            item['word_count'] = characteristics[0] if len(characteristics) else None

            parsed_el = xml.find('NewsItem/NewsComponent/RightsMetadata/UsageRights/UsageType')
            if parsed_el is not None:
                item.setdefault('usageterms', parsed_el.text)

            parsed_el = xml.findall('NewsItem/NewsComponent/DescriptiveMetadata/Genre')
            if parsed_el is not None:
                item['genre'] = []
                for el in parsed_el:
                    item['genre'].append({'name': el.get('FormalName')})

            return self.populate_fields(item)
        except Exception as ex:
            raise ParserError.newsmlOneParserError(ex, provider)

    def parse_elements(self, tree):
        items = {}
        for item in tree:
            if item.text is None:
                # read the attribute for the item
                if item.tag != 'HeadLine':
                    items[item.tag] = item.attrib
            else:
                # read the value for the item
                items[item.tag] = item.text
        return items

    def parse_multivalued_elements(self, tree):
        items = {}
        for item in tree:
            if item.tag not in items:
                items[item.tag] = [item.text]
            else:
                items[item.tag].append(item.text)

        return items

    def parse_attributes_as_dictionary(self, items):
        attributes = [item.attrib for item in items]
        return attributes

    def parse_attribute_values(self, items, attribute):
        attributes = []
        for item in items:
            if item.attrib['FormalName'] == attribute:
                attributes.append(item.attrib['Value'])
        return attributes

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, '%Y%m%dT%H%M%S+0000')
        except ValueError:
            return datetime.datetime.strptime(string, '%Y%m%dT%H%M%SZ').replace(tzinfo=utc)

    def populate_fields(self, item):
        item[ITEM_TYPE] = CONTENT_TYPE.TEXT
        return item

    def parse_news_identifier(self, item, tree):
        parsed_el = self.parse_elements(tree.find('NewsItem/Identification/NewsIdentifier'))
        item['guid'] = parsed_el['PublicIdentifier']
        item['version'] = parsed_el['RevisionId']

    def parse_news_management(self, item, tree):
        parsed_el = self.parse_elements(tree.find('NewsItem/NewsManagement'))
        item['urgency'] = int(parsed_el['Urgency']['FormalName'])
        item['versioncreated'] = self.datetime(parsed_el['ThisRevisionCreated'])
        item['firstcreated'] = self.datetime(parsed_el['FirstCreated'])
        item['pubstatus'] = (parsed_el['Status']['FormalName']).lower()
        # if parsed_el['NewsItemType']['FormalName'] == 'Alert':
        #    parsed_el['headline'] = 'Alert'

    def parse_newslines(self, item, tree):
        parsed_el = self.parse_elements(tree.find('NewsItem/NewsComponent/NewsLines'))

        self.set_dateline(item, text=parsed_el.get('DateLine', ''))

        item['headline'] = parsed_el.get('HeadLine', '')
        item['slugline'] = parsed_el.get('SlugLine', '')
        item['byline'] = parsed_el.get('ByLine', '')

        return True

    def format_subjects(self, subjects):
        """Map the ingested Subject Codes to their corresponding names as per IPTC Specification.

        :param subjects: list of dicts where each dict gives the category the article is mapped to.
        :type subjects: list
        :returns [{"qcode": "01001000", "name": "archaeology"}, {"qcode": "01002000", "name": "architecture"}]
        :rtype list
        """

        formatted_subjects = []

        def is_not_formatted(qcode):
            for formatted_subject in formatted_subjects:
                if formatted_subject['qcode'] == qcode:
                    return False

            return True

        for subject in subjects:
            formal_name = subject.get('FormalName')
            if formal_name and is_not_formatted(formal_name):
                formatted_subjects.append({'qcode': formal_name, 'name': subject_codes.get(formal_name, '')})

        return formatted_subjects


register_feed_parser(NewsMLOneFeedParser.NAME, NewsMLOneFeedParser())



import superdesk

from flask import current_app as app
from superdesk.utils import ListCursor
from superdesk.geonames import geonames_request, format_geoname_item


class PlacesAutocompleteResource(superdesk.Resource):

    resource_methods = ['GET']
    item_methods = []


class PlacesAutocompleteService(superdesk.Service):
    def get(self, req, lookup):
        assert req.args.get('name'), {'name': 1}
        params = [
            ('name_startsWith', req.args.get('name')),
            ('lang', req.args.get('lang')),
            ('style', req.args.get('style', 'medium')),
        ]

        if req.args.get('featureClass'):
            params.append(('featureClass', req.args.get('featureClass')))
        else:
            for feature_class in app.config['GEONAMES_FEATURE_CLASSES']:
                params.append(('featureClass', feature_class.upper()))

        json_data = geonames_request('search', params)
        data = [format_geoname_item(item) for item in json_data.get('geonames', [])]
        return ListCursor(data)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

__all__ = [
    'item_publish',
    'item_published',
    'item_update',
    'item_fetched',
    'item_moved',
]

import blinker

signals = blinker.Namespace()

#: Sent before item is published.
#:
#: .. versionadded:: 1.30
#:
#: :param sender: PublishService
#: :param item: item to publish
item_publish = signals.signal('item:publish')

#: Sent when item is published.
#:
#: :param sender: PublishService
#: :param item: published item
item_published = signals.signal('item:published')

#: Sent before new version is saved.
#:
#: :param sender: ArchiveService
#: :param updates: changes to be saved
#: :param original: original item version
item_update = signals.signal('item:update')

#: Sent after item is fetched.
#:
#: .. versionadded:: 1.29
#:
#: :param sender: FetchService
#: :param item: fetched item in production
#: :param ingest_item: item in ingest
item_fetched = signals.signal('item:fetched')

#: Sent after item is moved to different desk/stage.
#:
#: .. versionadded:: 1.29
#:
#: :param sender: MoveService
#: :param item: item after moving
#: :param original: item before moving
item_moved = signals.signal('item:moved')


#: Sent before item update is created
#:
#: .. versionadded:: 1.29
#:
#: :param sender: ArchiveRewriteService
#: :param item: new item update
#: :param original: original item
item_rewrite = signals.signal('item:rewrite')


#: Validate item
#:
#: You can add errors to response and that will
#: prevent publishing and display those errors to users.
#:
#: .. versionadded:: 1.30.1
#:
#: :param sender: ValidateService
#: :param item: item to validate
#: :param response: human readable list or errors
#: :param error_fields: system readable errors info
item_validate = signals.signal('item:validate')


def connect(signal, subscriber):
    """Connect to signal"""
    blinker.signal(signal).connect(subscriber)


def send(signal, sender, **kwargs):
    """Send signal"""
    return blinker.signal(signal).send(sender, **kwargs)


def proxy_resource_signal(action, app):
    def handle(resource, documents):
        docs = documents
        if '_items' in documents:
            docs = documents['_items']
        send(action, app.data, docs=docs)
        send('%s:%s' % (action, resource), app.data, docs=docs)
    return handle


def proxy_item_signal(action, app):
    def handle(resource, document):
        send(action, app.data, resource=resource, docs=[document])
        send('%s:%s' % (action, resource), app.data, docs=[document])
    return handle



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from setuptools import setup, find_packages

LONG_DESCRIPTION = "Superdesk Server Core"

install_requires = [
    'eve>=0.6,<=0.7.8',
    'eve-elastic==2.5.0',
    'flask>=0.10.1,<=0.12',
    'flask-oauthlib>=0.9.3,<0.10',
    'flask-mail>=0.9,<0.10',
    'flask-script>=2.0.5,<3.0',
    'flask_pymongo>=0.5.2,<1.0',
    'flask-babel==0.12.2',
    'pillow>=3.0,<=5.0',
    'arrow>=0.4,<=0.13',
    'asyncio>=3.4,<3.5',
    'bcrypt>=3.1.1,<3.2',
    'blinker>=1.3,<1.5',
    'celery[redis]>=4.3.0,<4.4',
    'redis>=3.2.0,<3.3',
    'kombu>=4.5,<4.6',
    'feedparser>=5.2,<5.3',
    'hachoir<=3.0a3',
    'HermesCache>=0.6.0,<0.8.0',
    'python-magic>=0.4,<0.5',
    'ldap3>=2.2.4,<2.6',
    'pytz>=2015.4',
    'tzlocal>=1.2.2,<2.0',
    'raven[flask]>=5.10,<7.0',
    'requests>=2.7.0,<3.0',
    'boto3>=1.1.4,<1.6',
    'websockets>=3.0,<7.0',
    'mongolock>=1.3.4,<1.4',
    'PyYAML>=4.2b1,<5.0',
    'lxml>=3.8,<4.2',
    'python-twitter==3.3',
    'chardet<4.0',
    'pymongo>=3.8,<3.9',
    'croniter<0.4',
    'python-dateutil<2.8',
    'unidecode==0.04.21',
]

package_data = {
    'superdesk': [
        'templates/*.txt',
        'templates/*.html',
        'locators/data/*.json',
        'io/data/*.json',
        'data_updates/*.py',
        'data_updates/*.js',
        'translations/*.po',
        'translations/*.mo',
    ],
    'apps': [
        'prepopulate/*.json',
        'prepopulate/data_init/*.json',
        'io/data/*.json',
    ],
}

setup(
    name='Superdesk-Core',
    version='1.32.1',
    description='Superdesk Core library',
    long_description=LONG_DESCRIPTION,
    author='petr jasek',
    author_email='petr.jasek@sourcefabric.org',
    url='https://github.com/superdesk/superdesk-core',
    license='GPLv3',
    platforms=['any'],
    packages=find_packages(exclude=['tests', 'features']),
    package_data=package_data,
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ]
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import unittest

from unittest import mock
from unittest.mock import MagicMock
from superdesk.tests import TestCase
from datetime import datetime, timedelta
from copy import deepcopy

from superdesk.io.commands.update_ingest import is_not_expired, process_iptc_codes, is_new_version


class FakeSuperdesk():
    def __init__(self):
        self.services = {}  # can be accessed and overriden by test methods

    def get_resource_service(self, service_name):
        return self.services.get(service_name)


fake_superdesk = FakeSuperdesk()


@mock.patch('superdesk.io.commands.update_ingest.superdesk', fake_superdesk)
class GetProviderRoutingSchemeTestCase(TestCase):
    """Tests for the get_provider_routing_scheme() function."""

    def setUp(self):
        try:
            from superdesk.io.commands.update_ingest import (
                get_provider_routing_scheme)
        except ImportError:
            self.fail("Could not import function under test " +
                      "(get_provider_routing_scheme).")
        else:
            self.funcToTest = get_provider_routing_scheme

        fake_superdesk.services = {
            'routing_schemes': MagicMock(name='routing_schemes'),
            'content_filters': MagicMock(name='content_filters')
        }

    def test_returns_none_if_no_provider_scheme_defined(self):
        fake_provider = {'routing_scheme': None}
        result = self.funcToTest(fake_provider)
        self.assertIsNone(result)

    def test_returns_scheme_config_from_db_if_scheme_defined(self):
        fake_scheme = {
            '_id': 'abc123',
            'rules': []
        }
        schemes_service = fake_superdesk.services['routing_schemes']
        schemes_service.find_one.return_value = fake_scheme

        fake_provider = {'routing_scheme': 'abc123'}
        result = self.funcToTest(fake_provider)

        # check that correct scheme has been fetched and returned
        self.assertTrue(schemes_service.find_one.called)
        args, kwargs = schemes_service.find_one.call_args
        self.assertEqual(kwargs.get('_id'), 'abc123')
        self.assertEqual(result, fake_scheme)

    def test_includes_content_filters_in_returned_scheme(self):
        fake_scheme = {
            '_id': 'abc123',
            'rules': [
                {'filter': 'filter_id_4'},
                {'filter': 'filter_id_8'},
            ]
        }
        schemes_service = fake_superdesk.services['routing_schemes']
        schemes_service.find_one.return_value = fake_scheme

        filters_service = fake_superdesk.services['content_filters']
        filters_service.find_one.side_effect = [
            {'_id': 'filter_id_4'},
            {'_id': 'filter_id_8'},
        ]

        fake_provider = {'routing_scheme': 'abc123'}
        result = self.funcToTest(fake_provider)

        scheme_rules = result.get('rules', [])
        self.assertEqual(len(scheme_rules), 2)
        self.assertEqual(scheme_rules[0].get('filter'), {'_id': 'filter_id_4'})
        self.assertEqual(scheme_rules[1].get('filter'), {'_id': 'filter_id_8'})


class ItemExpiryTestCase(TestCase):

    def test_expiry_no_dateinfo(self):
        self.assertTrue(is_not_expired({}, None))

    def test_expiry_overflow(self):
        item = {'versioncreated': datetime.now()}
        delta = timedelta(minutes=999999999999)
        self.assertTrue(is_not_expired(item, delta))


class IPTCCodesTestCase(TestCase):

    def test_unknown_iptc(self):
        """Test if an unknown IPTC code is not causing a crash"""
        item = {
            "guid": "urn:newsml:localhost:2019-02-07T12:00:00.030513:369c16e0-d6b7-40e1-8838-9c5f6a61626c",
            "subject": [{"name": "system", "qcode": "99009000"}],
        }
        # item should not be modified
        expected = deepcopy(item)

        with self.app.app_context():
            process_iptc_codes(item, {})
        self.assertEqual(item, expected)


class UtilsTestCase(unittest.TestCase):
    def test_is_new_version(self):
        self.assertTrue(is_new_version({'version': 2}, {'version': 1}))
        self.assertTrue(is_new_version({'versiocreated': datetime.now()},
                                       {'versioncreated': datetime.now() - timedelta(days=1)}))
        self.assertTrue(is_new_version({'version': '10'}, {'version': '2'}))

        self.assertFalse(is_new_version({'version': 1}, {'version': 1}))
        self.assertFalse(is_new_version({'version': '123'}, {'version': '123'}))
        self.assertFalse(is_new_version({'versioncreated': datetime.now()},
                                        {'versioncreated': datetime.now()}))

    def test_is_new_version_content(self):
        self.assertTrue(is_new_version({'headline': 'foo'}, {}))
        self.assertTrue(is_new_version({'headline': 'foo'}, {'headline': 'bar'}))
        self.assertTrue(is_new_version(
            {'renditions': {'original': {'href': 'foo'}}},
            {'renditions': {'original': {'href': 'bar'}}},
        ))
        self.assertTrue(is_new_version(
            {'subject': [{'name': 'foo', 'qcode': 'foo'}]},
            {'subject': [{'name': 'bar', 'qcode': 'bar'}]},
        ))

        self.assertFalse(is_new_version({}, {}))
        self.assertFalse(is_new_version({'headline': 'foo'}, {'headline': 'foo', 'source': 'test'}))
        self.assertFalse(is_new_version(
            {'renditions': {'original': {'href': 'foo'}}},
            {'renditions': {'original': {'href': 'foo'}}},
        ))
        self.assertFalse(is_new_version(
            {'subject': [{'name': 'foo', 'qcode': 'foo'}]},
            {'subject': [{'name': 'foo', 'qcode': 'foo'}]},
        ))

    def test_is_new_version_ignores_expiry(self):
        yesterday = datetime.now() - timedelta(days=1)
        self.assertFalse(is_new_version(
            {'headline': 'foo', 'firstcreated': None, 'expiry': datetime.now()},
            {'headline': 'foo', 'firstcreated': yesterday, 'expiry': yesterday},
        ))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.utc import utcnow
from email.utils import parsedate_to_datetime
from superdesk import etree as sd_etree
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_FIELD
from superdesk.media.renditions import update_renditions
from xml.sax.saxutils import quoteattr
from lxml import etree
from collections import OrderedDict

logger = logging.getLogger(__name__)
nsmap = {
    "excerpt": "http://wordpress.org/export/1.2/excerpt/",
    "content": "http://purl.org/rss/1.0/modules/content/",
    "wfw": "http://wellformedweb.org/CommentAPI/",
    "dc": "http://purl.org/dc/elements/1.1/",
    "wp": "http://wordpress.org/export/1.2/",
}
embed_TPL = "EMBED {} Image {{id: \"{}\"}}"

BLOCK_WHITELIST = {
    '<h',  # hr + h[1..9]
    '<br',
    '<p',
    '<div',
    '<iframe',
    '<ul',
    '<ol',
    '<li',
    '<table',
    '</',
}


def is_block_elem(line):
    text = line.lstrip()
    for block in BLOCK_WHITELIST:
        if text.startswith(block):
            return True
    return False


class WPWXRFeedParser(XMLFeedParser):
    """
    Feed Parser for Wordpress' WXR format
    """

    NAME = 'wpwxr'

    label = 'Wordpress WXR parser'

    def __init__(self):
        super().__init__()
        # we use OrderedDict to have control on parsing order
        self.default_mapping = OrderedDict([
            ('guid', 'guid'),
            ('item_id', 'guid'),
            ('_current_version', lambda _: 1),
            ('versioncreated', lambda _: utcnow()),
            ('firstpublished', {'xpath': 'pubDate/text()',
                                'filter': parsedate_to_datetime}),
            ('author', 'dc:creator'),
            ('headline', 'title'),
            # featuremedia must called before body_html, so thumbnail is used
            # in priority as feature media (else body_html would use first image
            # as feature media).
            ('featuremedia', self.parse_thumbnail),
            # images are handled in body_hook
            ('body_html', {'xpath': 'content:encoded',
                           'key_hook': self.body_hook}),
            ('keywords', {'xpath': 'category[@domain="post_tag"]',
                          'list': True}),
            ('anpa_category', {'xpath': 'category[@domain="category"]/text()',
                               'list': True,
                               'filter': lambda cat: {'qcode': cat, 'name': cat}}),
            ('attachments', {'xpath': 'wp:attachment_url',
                             'list': True,
                             'key_hook': self.attachments_hook})])

    def can_parse(self, xml):
        return xml.tag == 'rss'

    def parse(self, xml, provider=None):
        return list(self.parse_items(xml))

    def parse_items(self, xml):
        for item_xml in xml.findall('channel/item'):
            try:
                post_type = item_xml.xpath('wp:post_type', namespaces=nsmap)[0].text
            except IndexError:
                pass
            else:
                if post_type and post_type != "post":
                    # we don't want to parse attachments
                    continue
            yield self.parse_item(item_xml)

    def parse_item(self, item_xml):
        item = {}
        self.do_mapping(item, item_xml, namespaces=nsmap)
        if "associations" in item:
            for _, data in item['associations'].items():
                if data is None:
                    continue
                # these 3 fields are mandatory in default setup
                # we use a space for that to avoid issue when publishing
                data.setdefault('headline', item['headline'])
                data.setdefault('alt_text', ' ')
                data.setdefault('description_text', ' ')
            if (len(item['associations']) == 1 and not item['body_html'] and
               item.get('associations', {}).get('featuremedia')):
                # if the item only contains a feature media, we convert it to picture
                featuremedia = item['associations']['featuremedia']
                item['renditions'] = featuremedia['renditions']
                item[ITEM_TYPE] = featuremedia[ITEM_TYPE]
                item['alt_text'] = item['headline']
                item['media'] = item['renditions']['original']['media']
                item['mimetype'] = item['renditions']['original']['mimetype']
                del item['associations']
        return item

    def check_url(self, url):
        """Check URL and add protocol in case of relative URL

        :param url: found URL
        :return unicode: HTTP(s) URL
        :raises ValueError: the URL is invlalid
        """
        if url is None:
            raise ValueError("No URL found")
        url = url.strip()
        if not url:
            raise ValueError("URL is empty")
        if url.startswith('//'):
            # if we have a protocol relative URL, we use https
            url = "https:" + url
        if not url.startswith("http"):
            raise ValueError("Url is not HTTP(s)")
        return url

    def _add_image(self, item, url):
        associations = item.setdefault('associations', {})
        association = {
            ITEM_TYPE: CONTENT_TYPE.PICTURE,
            'ingest_provider': self.NAME,
            GUID_FIELD: url,
        }
        update_renditions(association, url, None)

        # we use featuremedia for the first image, then embeddedX
        if 'featuremedia' not in associations:
            key = 'featuremedia'
        else:
            key = 'embedded' + str(len(associations) - 1)

        associations[key] = association
        return key, association

    def parse_thumbnail(self, item_elt, item):
        """Check for _thumbnail_id meta_key, and use its attachment as feature media

        If the key is found, the linked item is looked for, and its attachment_url is used as feature media
        """
        thumbnail_elt = item_elt.xpath('wp:postmeta/wp:meta_key[text()="_thumbnail_id"]', namespaces=nsmap)
        if not thumbnail_elt:
            return
        thumbnail_elt = thumbnail_elt[0]

        try:
            post_id = thumbnail_elt.xpath("../wp:meta_value/text()", namespaces=nsmap)[0].strip()
            if not post_id:
                raise IndexError
        except IndexError:
            logger.warning("invalid post_id, ignoring: {elt}".format(
                elt=sd_etree.to_string(thumbnail_elt.xpath("..")[0])))
            return
        try:
            if '"' in post_id:
                raise ValueError('post id should not contain " (double quote)')
            post_id_elt = item_elt.xpath('/rss/channel/item/wp:post_id[text()="{}"]'
                                         .format(post_id), namespaces=nsmap)[0]
            att_item_elt = post_id_elt.getparent()
            url = att_item_elt.xpath('wp:attachment_url', namespaces=nsmap)[0].text.strip()
            url = self.check_url(url)
        except (IndexError, ValueError) as e:
            logger.warning("Can't find attachement URL, ignoring: {e}\n{elt}".format(
                e=e, elt=sd_etree.to_string(thumbnail_elt.getparent())))
            return
        try:
            key, media_data = self._add_image(item, url)
        except Exception as e:
            logger.error(e)
            return

        for key, elt_names in (("description_text", ("description", "title")),
                               ("alt_text", ("title",))):
            for elt_name in elt_names:
                elt = att_item_elt.find(elt_name)
                if elt is not None and elt.text:
                    media_data[key] = elt.text
                    break
            else:
                media_data[key] = ""

    def body_hook(self, item, html):
        """Copy content to body_html

        if img are found in the content, they are uploaded.
        First image is used as feature media, then there are embeds
        """
        # we need to convert CRLF to <p>
        # cf. SDTS-22
        html = html.replace('&#13;', '\r')
        splitted = html.split('\r\n')
        if len(splitted) == 1 and '<p>' not in html:
            splitted = html.split('\n')
        if len(splitted) > 1:
            html = ''.join(['<p>{}</p>'.format(s) if not is_block_elem(s) else s for s in splitted if s.strip()])

        if "img" in html:
            content = sd_etree.parse_html(html, 'html')
            for img in content.xpath('//img'):
                try:
                    src = self.check_url(img.get('src'))
                except ValueError:
                    logger.warning("Can't fetch image: {elt}".format(elt=sd_etree.to_string(img)))
                    continue
                try:
                    key, media_data = self._add_image(item, src)
                except Exception as e:
                    logger.error(e)
                    img.getparent().remove(img)
                    continue
                url = media_data['renditions']['original']['href']
                img.set("src", url)
                if key == 'featuremedia':
                    # no need to embed the image for featuremedia
                    continue
                embed_start = etree.Comment(embed_TPL.format('START', key))
                embed_end = etree.Comment(embed_TPL.format('END', key))
                img.addprevious(embed_start)
                img.addnext(embed_end)

            content = sd_etree.fix_html_void_elements(content)
            html = sd_etree.to_string(content, encoding="unicode", method='xml')

        item['body_html'] = html

    def attachments_hook(self, item, attachments):
        """Attachment are parsed at the end

        if it's the first image found, it's used as feature media
        else it's used as embed and put at the end of body_html
        """
        for url in attachments:
            try:
                key, media_data = self._add_image(item, url)
            except Exception as e:
                logger.error(e)
                continue
            if key == 'featuremedia':
                # no need to embed the image for featuremedia
                continue
            embed_start = "<!--" + embed_TPL.format('START', key) + "-->"
            embed_end = "<!--" + embed_TPL.format('END', key) + "-->"
            new_url = media_data['renditions']['original']['href']
            img = '<img src={src} height="{height}" width="{width}">'.format(
                src=quoteattr(new_url),
                height=media_data['renditions']['original']['height'],
                width=media_data['renditions']['original']['width'])
            item['body_html'] += '<div>' + embed_start + img + embed_end + '</div>'


register_feed_parser(WPWXRFeedParser.NAME, WPWXRFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .workspace import WorkspaceService, WorkspaceResource


def init_app(app):
    superdesk.register_resource('workspaces', WorkspaceResource, WorkspaceService,
                                privilege=['POST', 'PATCH', 'DELETE'])

    superdesk.register_default_user_preference('workspace:active', {
        'type': 'string',
        'workspace': ''
    })

    superdesk.privilege(name='dashboard',
                        label='Dashboard',
                        description='Can access dashboard.')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import datetime

from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.errors import ParserError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE
from superdesk.utc import utc


class WENNFeedParser(XMLFeedParser):
    """
    Feed Parser for parsing the XML supplied by WENN
    """

    NAME = 'wenn'

    label = 'WENN Parser'

    ATOM_NS = 'http://www.w3.org/2005/Atom'
    WENN_MM_NS = 'http://feed.wenn.com/xmlns/2010/03/wenn_mm'
    WENN_NM_NS = 'http://feed.wenn.com/xmlns/2010/03/wenn_nm'
    WENN_CM_NS = 'http://feed.wenn.com/xmlns/2010/03/wenn_cm'
    GEORSS_NS = 'http://www.georss.org/georss'

    def can_parse(self, xml):
        return xml.tag == self.qname('feed', self.ATOM_NS) and len(xml.findall(
            self.qname('NewsManagement', self.WENN_NM_NS))) > 0

    def parse(self, xml, provider=None):
        itemList = []
        try:
            for entry in xml.findall(self.qname('entry', self.ATOM_NS)):
                item = {}
                self.set_item_defaults(item)
                self.parse_content_management(item, entry)
                self.parse_news_management(item, entry)
                item['body_html'] = self.get_elem_content(entry.find(self.qname('content', self.ATOM_NS)))
                item['body_html'] = item['body_html'].replace('\n\n  ', '</p><p>').replace('\n', '<br>')
                item['body_html'] = '<p>' + item['body_html'] + '</p>'
                itemList.append(item)
            return itemList

        except Exception as ex:
            raise ParserError.wennParserError(ex, provider)

    def set_item_defaults(self, item):
        item[ITEM_TYPE] = CONTENT_TYPE.TEXT
        item['urgency'] = 5
        item['pubstatus'] = 'usable'
        item['anpa_category'] = [{'qcode': 'e'}]
        item['subject'] = [{'qcode': '01000000', 'name': 'arts, culture and entertainment'}]

    def parse_news_management(self, item, entry):
        news_mgmt_el = entry.find(self.qname('NewsManagement', self.WENN_NM_NS))
        if news_mgmt_el is not None:
            item['firstcreated'] = self.datetime(self.get_elem_content(
                news_mgmt_el.find(self.qname('published', self.WENN_NM_NS))))
            item['versioncreated'] = self.datetime(self.get_elem_content(
                news_mgmt_el.find(self.qname('updated', self.WENN_NM_NS))))
            item['guid'] = self.get_elem_content(
                news_mgmt_el.find(self.qname('original_article_id', self.WENN_NM_NS)))

    def parse_content_management(self, item, entry):
        content_mgmt_el = entry.find(self.qname('ContentMetadata', self.WENN_CM_NS))
        if content_mgmt_el is not None:
            item['headline'] = self.get_elem_content(content_mgmt_el.find(self.qname('title', self.WENN_CM_NS)))
            item['abstract'] = self.get_elem_content(
                content_mgmt_el.find(self.qname('first_line', self.WENN_CM_NS)))

    def get_elem_content(self, elem):
        return elem.text if elem is not None else ''

    def datetime(self, string):
        return datetime.datetime.strptime(string, '%Y-%m-%dT%H:%M:%S+00:00').replace(tzinfo=utc)


register_feed_parser(WENNFeedParser.NAME, WENNFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.media.crop import CropService
from superdesk.metadata.item import ITEM_STATE, EMBARGO, SCHEDULE_SETTINGS
from superdesk.utc import utcnow
from superdesk.text_utils import update_word_count
from apps.archive.common import set_sign_off, ITEM_OPERATION
from apps.archive.archive import update_associations, flush_renditions
from .common import BasePublishService, BasePublishResource, ITEM_CORRECT
from superdesk.emails import send_translation_changed
from superdesk.activity import add_activity
from flask import g


def send_translation_notifications(original):
    translated_items = get_resource_service('published').find({
        'translation_id': original.get('guid'),
        'last_published_version': True
    })

    user_ids = set()
    for translated_item in translated_items:
        if translated_item.get('item_id') == original.get('_id'):
            changed_article = translated_item
            continue
        user_ids.add(translated_item.get('original_creator'))
        user_ids.add(translated_item.get('version_creator'))

    if len(user_ids) == 0 or changed_article is None:
        return

    add_activity('translated:changed', '', resource=None, item=changed_article, notify=user_ids)

    recipients = []
    for user_id in user_ids:
        user = get_resource_service('users').find_one(req=None, _id=user_id)
        if user is not None and user.get('email', None) is not None:
            recipients.append(user.get('email'))

    if len(recipients) == 0:
        return

    username = g.user.get('display_name') or g.user.get('username')
    send_translation_changed(username, changed_article, recipients)


class CorrectPublishResource(BasePublishResource):

    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, 'correct')


class CorrectPublishService(BasePublishService):
    publish_type = 'correct'
    published_state = 'corrected'
    item_operation = ITEM_CORRECT

    def set_state(self, original, updates):
        updates[ITEM_STATE] = self.published_state

        if original.get(EMBARGO) or updates.get(EMBARGO):
            # embargo time elapsed
            utc_embargo = updates.get(SCHEDULE_SETTINGS, {}).get('utc_{}'.format(EMBARGO)) or \
                original.get(SCHEDULE_SETTINGS, {}).get('utc_{}'.format(EMBARGO))
            if utc_embargo and utc_embargo < utcnow():
                # remove embargo information. so the next correction is without embargo.
                updates[EMBARGO] = None
                super().set_state(original, updates)
        else:
            super().set_state(original, updates)

    def on_update(self, updates, original):
        update_associations(updates)
        CropService().validate_multiple_crops(updates, original)
        super().on_update(updates, original)
        updates[ITEM_OPERATION] = self.item_operation
        updates['versioncreated'] = utcnow()
        updates['correction_sequence'] = original.get('correction_sequence', 1) + 1
        set_sign_off(updates, original)
        update_word_count(updates, original)
        flush_renditions(updates, original)

    def update(self, id, updates, original):
        CropService().create_multiple_crops(updates, original)
        super().update(id, updates, original)

    def on_updated(self, updates, original):
        super().on_updated(updates, original)
        send_translation_notifications(original)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from apps.search_providers.registry import registered_search_providers, allowed_search_providers, register_search_provider  # noqa
from apps.search_providers.resource import SearchProviderResource
from apps.search_providers.service import SearchProviderService
from apps.search_providers.registry import SearchProviderAllowedResource, SearchProviderAllowedService


def init_app(app):
    from apps.search_providers.proxy import SearchProviderProxyResource, SearchProviderProxyService
    superdesk.privilege(
        name='search_providers',
        label='Manage Search Providers',
        description='User can manage search providers.'
    )

    superdesk.register_resource(
        name='search_providers',
        resource=SearchProviderResource,
        service=SearchProviderService
    )

    superdesk.register_resource(
        name='search_providers_proxy',
        resource=SearchProviderProxyResource,
        service=SearchProviderProxyService
    )

    superdesk.register_resource(
        name='search_providers_allowed',
        resource=SearchProviderAllowedResource,
        service=SearchProviderAllowedService
    )
